# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rservice.proto\x12\x06klotho\"\x1d\n\tIRRequest\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\"0\n\x07IRReply\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x14\n\x0cyaml_payload\x18\x02 \x01(\t\"\x14\n\x12HealthCheckRequest\"\"\n\x10HealthCheckReply\x12\x0e\n\x06status\x18\x01 \x01(\t\"0\n\x18RegisterConstructRequest\x12\x14\n\x0cyaml_payload\x18\x01 \x01(\t\"?\n\x16RegisterConstructReply\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x14\n\x0cyaml_payload\x18\x02 \x01(\t2\xdf\x01\n\rKlothoService\x12.\n\x06SendIR\x12\x11.klotho.IRRequest\x1a\x0f.klotho.IRReply\"\x00\x12\x45\n\x0bHealthCheck\x12\x1a.klotho.HealthCheckRequest\x1a\x18.klotho.HealthCheckReply\"\x00\x12W\n\x11RegisterConstruct\x12 .klotho.RegisterConstructRequest\x1a\x1e.klotho.RegisterConstructReply\"\x00\x42:Z8github.com/klothoplatform/klotho/pkg/k2/language_host/gob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z8github.com/klothoplatform/klotho/pkg/k2/language_host/go'
  _globals['_IRREQUEST']._serialized_start=25
  _globals['_IRREQUEST']._serialized_end=54
  _globals['_IRREPLY']._serialized_start=56
  _globals['_IRREPLY']._serialized_end=104
  _globals['_HEALTHCHECKREQUEST']._serialized_start=106
  _globals['_HEALTHCHECKREQUEST']._serialized_end=126
  _globals['_HEALTHCHECKREPLY']._serialized_start=128
  _globals['_HEALTHCHECKREPLY']._serialized_end=162
  _globals['_REGISTERCONSTRUCTREQUEST']._serialized_start=164
  _globals['_REGISTERCONSTRUCTREQUEST']._serialized_end=212
  _globals['_REGISTERCONSTRUCTREPLY']._serialized_start=214
  _globals['_REGISTERCONSTRUCTREPLY']._serialized_end=277
  _globals['_KLOTHOSERVICE']._serialized_start=280
  _globals['_KLOTHOSERVICE']._serialized_end=503
# @@protoc_insertion_point(module_scope)
